@@ -18,9 +18,17 @@
 from . import mparser, mesonlib, mlog
 from . import environment, dependencies
 
-import os, copy, re, types
+import os, copy, re
 from functools import wraps
 
+class ObjectHolder:
+    def __init__(self, obj, subproject=None):
+        self.held_object = obj
+        self.subproject = subproject
+
+    def __repr__(self):
+        return '<Holder: {!r}>'.format(self.held_object)
+
 # Decorators for method calls.
 
 def check_stringlist(a, msg='Arguments must be strings.'):
@@ -39,14 +47,14 @@ def _get_callee_args(wrapped_args, want_subproject=False):
     if want_subproject and n == 2:
         if hasattr(s, 'subproject'):
             # Interpreter base types have 2 args: self, node
-            node_or_state = wrapped_args[1]
+            node = wrapped_args[1]
             # args and kwargs are inside the node
             args = None
             kwargs = None
             subproject = s.subproject
         elif hasattr(wrapped_args[1], 'subproject'):
             # Module objects have 2 args: self, interpreter
-            node_or_state = wrapped_args[1]
+            node = wrapped_args[1].current_node
             # args and kwargs are inside the node
             args = None
             kwargs = None
@@ -55,7 +63,7 @@ def _get_callee_args(wrapped_args, want_subproject=False):
             raise AssertionError('Unknown args: {!r}'.format(wrapped_args))
     elif n == 3:
         # Methods on objects (*Holder, MesonMain, etc) have 3 args: self, args, kwargs
-        node_or_state = None # FIXME
+        node = s.current_node
         args = wrapped_args[1]
         kwargs = wrapped_args[2]
         if want_subproject:
@@ -65,30 +73,32 @@ def _get_callee_args(wrapped_args, want_subproject=False):
                 subproject = s.interpreter.subproject
     elif n == 4:
         # Meson functions have 4 args: self, node, args, kwargs
-        # Module functions have 4 args: self, state, args, kwargs; except,
-        # PythonInstallation methods have self, interpreter, args, kwargs
-        node_or_state = wrapped_args[1]
+        # Module functions have 4 args: self, state, args, kwargs
+        if isinstance(s, InterpreterBase):
+            node = wrapped_args[1]
+        else:
+            node = wrapped_args[1].current_node
         args = wrapped_args[2]
         kwargs = wrapped_args[3]
         if want_subproject:
             if isinstance(s, InterpreterBase):
                 subproject = s.subproject
             else:
-                subproject = node_or_state.subproject
+                subproject = wrapped_args[1].subproject
     elif n == 5:
         # Module snippets have 5 args: self, interpreter, state, args, kwargs
-        node_or_state = wrapped_args[2]
+        node = wrapped_args[2].current_node
         args = wrapped_args[3]
         kwargs = wrapped_args[4]
         if want_subproject:
-            subproject = node_or_state.subproject
+            subproject = wrapped_args[2].subproject
     else:
         raise AssertionError('Unknown args: {!r}'.format(wrapped_args))
     # Sometimes interpreter methods are called internally with None instead of
     # empty list/dict
     args = args if args is not None else []
     kwargs = kwargs if kwargs is not None else {}
-    return s, node_or_state, args, kwargs, subproject
+    return s, node, args, kwargs, subproject
 
 def flatten(args):
     if isinstance(args, mparser.StringNode):
@@ -134,9 +144,20 @@ def stringArgs(f):
     return wrapped
 
 def noArgsFlattening(f):
-    setattr(f, 'no-args-flattening', True)
+    setattr(f, 'no-args-flattening', True)  # noqa: B010
     return f
 
+def disablerIfNotFound(f):
+    @wraps(f)
+    def wrapped(*wrapped_args, **wrapped_kwargs):
+        kwargs = _get_callee_args(wrapped_args)[3]
+        disabler = kwargs.pop('disabler', False)
+        ret = f(*wrapped_args, **wrapped_kwargs)
+        if disabler and not ret.held_object.found():
+            return Disabler()
+        return ret
+    return wrapped
+
 class permittedKwargs:
 
     def __init__(self, permitted):
@@ -145,19 +166,10 @@ class permittedKwargs:
     def __call__(self, f):
         @wraps(f)
         def wrapped(*wrapped_args, **wrapped_kwargs):
-            s, node_or_state, args, kwargs, _ = _get_callee_args(wrapped_args)
-            loc = types.SimpleNamespace()
-            if hasattr(s, 'subdir'):
-                loc.subdir = s.subdir
-                loc.lineno = s.current_lineno
-            elif node_or_state and hasattr(node_or_state, 'subdir'):
-                loc.subdir = node_or_state.subdir
-                loc.lineno = node_or_state.current_lineno
-            else:
-                loc = None
+            s, node, args, kwargs, _ = _get_callee_args(wrapped_args)
             for k in kwargs:
                 if k not in self.permitted:
-                    mlog.warning('''Passed invalid keyword argument "{}".'''.format(k), location=loc)
+                    mlog.warning('''Passed invalid keyword argument "{}".'''.format(k), location=node)
                     mlog.warning('This will become a hard error in the future.')
             return f(*wrapped_args, **wrapped_kwargs)
         return wrapped
@@ -292,9 +304,18 @@ class InvalidArguments(InterpreterException):
 class SubdirDoneRequest(BaseException):
     pass
 
+class ContinueRequest(BaseException):
+    pass
+
+class BreakRequest(BaseException):
+    pass
+
 class InterpreterObject:
     def __init__(self):
         self.methods = {}
+        # Current node set during a method call. This can be used as location
+        # when printing a warning message during a method call.
+        self.current_node = None
 
     def method_call(self, method_name, args, kwargs):
         if method_name in self.methods:
@@ -316,20 +337,25 @@ class Disabler(InterpreterObject):
     def found_method(self, args, kwargs):
         return False
 
-def is_disabler(i):
+def is_disabler(i) -> bool:
     return isinstance(i, Disabler)
 
-def is_disabled(args, kwargs):
+def is_arg_disabled(arg) -> bool:
+    if is_disabler(arg):
+        return True
+    if isinstance(arg, list):
+        for i in arg:
+            if is_arg_disabled(i):
+                return True
+    return False
+
+def is_disabled(args, kwargs) -> bool:
     for i in args:
-        if isinstance(i, Disabler):
+        if is_arg_disabled(i):
             return True
     for i in kwargs.values():
-        if isinstance(i, Disabler):
+        if is_arg_disabled(i):
             return True
-        if isinstance(i, list):
-            for j in i:
-                if isinstance(j, Disabler):
-                    return True
     return False
 
 class InterpreterBase:
@@ -341,6 +367,9 @@ class InterpreterBase:
         self.variables = {}
         self.argument_depth = 0
         self.current_lineno = -1
+        # Current node set during a function call. This can be used as location
+        # when printing a warning message during a method call.
+        self.current_node = None
 
     def load_root_meson_file(self):
         mesonfile = os.path.join(self.source_root, self.subdir, environment.build_filename)
@@ -357,6 +386,9 @@ class InterpreterBase:
             me.file = environment.build_filename
             raise me
 
+    def join_path_strings(self, args):
+        return os.path.join(*args).replace('\\', '/')
+
     def parse_project(self):
         """
         Parses project() and initializes languages, compilers etc. Do this
@@ -445,6 +477,10 @@ class InterpreterBase:
             return self.evaluate_indexing(cur)
         elif isinstance(cur, mparser.TernaryNode):
             return self.evaluate_ternary(cur)
+        elif isinstance(cur, mparser.ContinueNode):
+            raise ContinueRequest()
+        elif isinstance(cur, mparser.BreakNode):
+            raise BreakRequest()
         elif self.is_elementary_type(cur):
             return cur
         else:
@@ -487,6 +523,13 @@ class InterpreterBase:
             return False
         return True
 
+    def evaluate_in(self, val1, val2):
+        if not isinstance(val1, (str, int, float, ObjectHolder)):
+            raise InvalidArguments('lvalue of "in" operator must be a string, integer, float, or object')
+        if not isinstance(val2, (list, dict)):
+            raise InvalidArguments('rvalue of "in" operator must be an array or a dict')
+        return val1 in val2
+
     def evaluate_comparison(self, node):
         val1 = self.evaluate_statement(node.left)
         if is_disabler(val1):
@@ -494,6 +537,10 @@ class InterpreterBase:
         val2 = self.evaluate_statement(node.right)
         if is_disabler(val2):
             return val2
+        if node.ctype == 'in':
+            return self.evaluate_in(val1, val2)
+        elif node.ctype == 'notin':
+            return not self.evaluate_in(val1, val2)
         valid = self.validate_comparison_types(val1, val2)
         # Ordering comparisons of different types isn't allowed since PR #1810
         # (0.41.0).  Since PR #2884 we also warn about equality comparisons of
@@ -564,6 +611,23 @@ The result of this is undefined and will become a hard error in a future Meson r
             raise InterpreterException('Argument to negation is not an integer.')
         return -v
 
+    @FeatureNew('/ with string arguments', '0.49.0')
+    def evaluate_path_join(self, l, r):
+        if not isinstance(l, str):
+            raise InvalidCode('The division operator can only append to a string.')
+        if not isinstance(r, str):
+            raise InvalidCode('The division operator can only append a string.')
+        return self.join_path_strings((l, r))
+
+    def evaluate_division(self, l, r):
+        if isinstance(l, str) or isinstance(r, str):
+            return self.evaluate_path_join(l, r)
+        if isinstance(l, int) and isinstance(r, int):
+            if r == 0:
+                raise InvalidCode('Division by zero.')
+            return l // r
+        raise InvalidCode('Division works only with strings or integers.')
+
     def evaluate_arithmeticstatement(self, cur):
         l = self.evaluate_statement(cur.left)
         if is_disabler(l):
@@ -573,6 +637,8 @@ The result of this is undefined and will become a hard error in a future Meson r
             return r
 
         if cur.operation == 'add':
+            if isinstance(l, dict) and isinstance(r, dict):
+                return {**l, **r}
             try:
                 return l + r
             except Exception as e:
@@ -586,9 +652,7 @@ The result of this is undefined and will become a hard error in a future Meson r
                 raise InvalidCode('Multiplication works only with integers.')
             return l * r
         elif cur.operation == 'div':
-            if not isinstance(l, int) or not isinstance(r, int):
-                raise InvalidCode('Division works only with integers.')
-            return l // r
+            return self.evaluate_division(l, r)
         elif cur.operation == 'mod':
             if not isinstance(l, int) or not isinstance(r, int):
                 raise InvalidCode('Modulo works only with integers.')
@@ -620,7 +684,12 @@ The result of this is undefined and will become a hard error in a future Meson r
                 return items
             for item in items:
                 self.set_variable(varname, item)
-                self.evaluate_codeblock(node.block)
+                try:
+                    self.evaluate_codeblock(node.block)
+                except ContinueRequest:
+                    continue
+                except BreakRequest:
+                    break
         elif isinstance(items, dict):
             if len(node.varnames) != 2:
                 raise InvalidArguments('Foreach on dict unpacks key and value')
@@ -629,7 +698,12 @@ The result of this is undefined and will become a hard error in a future Meson r
             for key, value in items.items():
                 self.set_variable(node.varnames[0].value, key)
                 self.set_variable(node.varnames[1].value, value)
-                self.evaluate_codeblock(node.block)
+                try:
+                    self.evaluate_codeblock(node.block)
+                except ContinueRequest:
+                    continue
+                except BreakRequest:
+                    break
         else:
             raise InvalidArguments('Items of foreach loop must be an array or a dict')
 
@@ -651,14 +725,18 @@ The result of this is undefined and will become a hard error in a future Meson r
             if not isinstance(addition, int):
                 raise InvalidArguments('The += operator requires an int on the right hand side if the variable on the left is an int')
             new_value = old_variable + addition
-        elif not isinstance(old_variable, list):
-            raise InvalidArguments('The += operator currently only works with arrays, strings or ints ')
-        # Add other data types here.
-        else:
+        elif isinstance(old_variable, list):
             if isinstance(addition, list):
                 new_value = old_variable + addition
             else:
                 new_value = old_variable + [addition]
+        elif isinstance(old_variable, dict):
+            if not isinstance(addition, dict):
+                raise InvalidArguments('The += operator requires a dict on the right hand side if the variable on the left is a dict')
+            new_value = {**old_variable, **addition}
+        # Add other data types here.
+        else:
+            raise InvalidArguments('The += operator currently only works with arrays, dicts, strings or ints ')
         self.set_variable(varname, new_value)
 
     def evaluate_indexing(self, node):
@@ -689,13 +767,14 @@ The result of this is undefined and will become a hard error in a future Meson r
     def function_call(self, node):
         func_name = node.func_name
         (posargs, kwargs) = self.reduce_arguments(node.args)
-        if is_disabled(posargs, kwargs):
+        if is_disabled(posargs, kwargs) and func_name != 'set_variable' and func_name != 'is_disabler':
             return Disabler()
         if func_name in self.funcs:
             func = self.funcs[func_name]
             if not getattr(func, 'no-args-flattening', False):
                 posargs = flatten(posargs)
 
+            self.current_node = node
             return func(node, posargs, kwargs)
         else:
             self.unknown_function_called(func_name)
@@ -732,6 +811,7 @@ The result of this is undefined and will become a hard error in a future Meson r
             return Disabler()
         if method_name == 'extract_objects':
             self.validate_extraction(obj.held_object)
+        obj.current_node = node
         return obj.method_call(method_name, args, kwargs)
 
     def bool_method_call(self, obj, method_name, args):
@@ -845,6 +925,24 @@ The result of this is undefined and will become a hard error in a future Meson r
             return mesonlib.version_compare(obj, cmpr)
         raise InterpreterException('Unknown method "%s" for a string.' % method_name)
 
+    def format_string(self, templ, args):
+        if isinstance(args, mparser.ArgumentNode):
+            args = args.arguments
+        arg_strings = []
+        for arg in args:
+            arg = self.evaluate_statement(arg)
+            if isinstance(arg, bool): # Python boolean is upper case.
+                arg = str(arg).lower()
+            arg_strings.append(str(arg))
+
+        def arg_replace(match):
+            idx = int(match.group(1))
+            if idx >= len(arg_strings):
+                raise InterpreterException('Format placeholder @{}@ out of range.'.format(idx))
+            return arg_strings[idx]
+
+        return re.sub(r'@(\d+)@', arg_replace, templ)
+
     def unknown_function_called(self, func_name):
         raise InvalidCode('Unknown function "%s".' % func_name)
 
@@ -927,7 +1025,22 @@ The result of this is undefined and will become a hard error in a future Meson r
             a = args.kwargs[key]
             reduced_kw[key] = self.evaluate_statement(a)
         self.argument_depth -= 1
-        return reduced_pos, reduced_kw
+        final_kw = self.expand_default_kwargs(reduced_kw)
+        return reduced_pos, final_kw
+
+    def expand_default_kwargs(self, kwargs):
+        if 'kwargs' not in kwargs:
+            return kwargs
+        to_expand = kwargs.pop('kwargs')
+        if not isinstance(to_expand, dict):
+            raise InterpreterException('Value of "kwargs" must be dictionary.')
+        if 'kwargs' in to_expand:
+            raise InterpreterException('Kwargs argument must not contain a "kwargs" entry. Points for thinking meta, though. :P')
+        for k, v in to_expand.items():
+            if k in kwargs:
+                raise InterpreterException('Entry "{}" defined both as a keyword argument and in a "kwarg" entry.'.format(k))
+            kwargs[k] = v
+        return kwargs
 
     def assignment(self, node):
         assert(isinstance(node, mparser.AssignmentNode))
