@@ -41,14 +41,6 @@ mod_kwargs = set(['subdir'])
 mod_kwargs.update(known_shmod_kwargs)
 mod_kwargs -= set(['name_prefix', 'name_suffix'])
 
-
-def run_command(python, command):
-    cmd = python.get_command() + ['-c', command]
-    _, stdout, _ = mesonlib.Popen_safe(cmd)
-
-    return stdout.strip()
-
-
 class PythonDependency(ExternalDependency):
 
     def __init__(self, python_holder, environment, kwargs):
@@ -123,7 +115,6 @@ class PythonDependency(ExternalDependency):
                     self._find_libpy_windows(environment)
                 else:
                     self._find_libpy(python_holder, environment)
-
                 if self.is_found:
                     mlog.debug('Found "python-{}" via SYSCONFIG module'.format(self.version))
                     py_lookup_method = 'sysconfig'
@@ -131,7 +122,7 @@ class PythonDependency(ExternalDependency):
         if self.is_found:
             mlog.log('Dependency', mlog.bold(self.name), 'found:', mlog.green('YES ({})'.format(py_lookup_method)))
         else:
-            mlog.log('Dependency', mlog.bold(self.name), 'found:', [mlog.red('NO')])
+            mlog.log('Dependency', mlog.bold(self.name), 'found:', mlog.red('NO'))
 
     def _find_libpy(self, python_holder, environment):
         if python_holder.is_pypy:
@@ -153,7 +144,7 @@ class PythonDependency(ExternalDependency):
         if largs is not None:
             self.link_args = largs
 
-        self.is_found = largs is not None or not self.link_libpython
+        self.is_found = largs is not None or self.link_libpython
 
         inc_paths = mesonlib.OrderedSet([
             self.variables.get('INCLUDEPY'),
@@ -568,9 +559,16 @@ class PythonModule(ExtensionModule):
         else:
             # Sanity check, we expect to have something that at least quacks in tune
             try:
-                info = json.loads(run_command(python, INTROSPECT_COMMAND))
+                cmd = python.get_command() + ['-c', INTROSPECT_COMMAND]
+                p, stdout, stderr = mesonlib.Popen_safe(cmd)
+                info = json.loads(stdout)
             except json.JSONDecodeError:
                 info = None
+                mlog.debug('Could not introspect Python (%s): exit code %d' % (str(p.args), p.returncode))
+                mlog.debug('Program stdout:\n')
+                mlog.debug(stdout)
+                mlog.debug('Program stderr:\n')
+                mlog.debug(stderr)
 
             if isinstance(info, dict) and 'version' in info and self._check_version(name_or_path, info['version']):
                 res = PythonInstallation(interpreter, python, info)
